#include "peripheral/nvm/plib_nvm.h"
#include <sys/kmem.h>


/*******************************************************************************
/*
  Function:
    uint32_t virtualToPhysical (uint32_t address)

  Summary:
    Converts a virtual memory address to a physical one
 */
uint32_t virtualToPhysical(uint32_t address) {
    return (address & 0x1FFFFFFF);
}

/*******************************************************************************
/*
  Function:
    void NVMpageErase (uint32_t address)

  Summary:
    Erases a page in flash memory (16KB)
 */
void NVMpageErase(uint32_t address) {
    // Base address of page to be erased
    PLIB_NVM_FlashAddressToModify(NVM_ID_0, virtualToPhysical(address));

    // Disable flash write/erase operations
    PLIB_NVM_MemoryModifyInhibit(NVM_ID_0);

    // Select page erase function & enable flash write/erase operations
    PLIB_NVM_MemoryOperationSelect(NVM_ID_0, PAGE_ERASE_OPERATION);

    // Allow memory modifications
    PLIB_NVM_MemoryModifyEnable(NVM_ID_0);

    // Write the unlock key sequence
    PLIB_NVM_FlashWriteKeySequence(NVM_ID_0, 0x0);
    PLIB_NVM_FlashWriteKeySequence(NVM_ID_0, 0xAA996655);
    PLIB_NVM_FlashWriteKeySequence(NVM_ID_0, 0x556699AA);

    // Start the operation
    PLIB_NVM_FlashWriteStart(NVM_ID_0);

    // Wait until the operation has completed
    while (!PLIB_NVM_FlashWriteCycleHasCompleted(NVM_ID_0));

    // Disable flash write/erase operations
    PLIB_NVM_MemoryModifyInhibit(NVM_ID_0);

}

/*******************************************************************************
/*
  Function:
    void NVMwriteWord(uint32_t address, uint32_t data)

  Summary:
    Writes a word in flash memory (4 Bytes)
 */
void NVMwriteWord(uint32_t destAddr, uint32_t data) {
    // Base address of row to be written to (destination)
    PLIB_NVM_FlashAddressToModify(NVM_ID_0, virtualToPhysical(destAddr));

    // Set Datum
    PLIB_NVM_FlashProvideData(NVM_ID_0, data);

    // Disable flash write/erase operations
    PLIB_NVM_MemoryModifyInhibit(NVM_ID_0);

    // Select row write function & enable flash write/erase operations
    PLIB_NVM_MemoryOperationSelect(NVM_ID_0, WORD_PROGRAM_OPERATION);

    // Allow memory modifications
    PLIB_NVM_MemoryModifyEnable(NVM_ID_0);

    // Write the unlock key sequence
    PLIB_NVM_FlashWriteKeySequence(NVM_ID_0, 0xAA996655);
    PLIB_NVM_FlashWriteKeySequence(NVM_ID_0, 0x556699AA);

    // Start the operation
    PLIB_NVM_FlashWriteStart(NVM_ID_0);

    // Wait until the operation has completed
    while (!PLIB_NVM_FlashWriteCycleHasCompleted(NVM_ID_0));

    // Disable flash write/erase operations
    PLIB_NVM_MemoryModifyInhibit(NVM_ID_0);

}
